# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     geno
   Description :
   Author :        Asdil
   date：          2019/5/8
-------------------------------------------------
   Change Activity:
                   2019/5/8:
-------------------------------------------------
"""
__author__ = 'Asdil'
import pandas as pd
import gzip

# 识别过滤的行数
def skipRow(path, key):
    skip = 0
    with open(path, 'r') as f:
        for line in f:
            if key == line[:len(key)]:
                return skip
            skip += 1
    return skip


# 读取call文件
def readCall(path, key='probeset_id'):
    df = pd.read_table(path, skiprows=skipRow(path, key), low_memory=False)
    return df


# 读取vcf
def readVcf(path, key='#CHROM'):
    df = pd.read_table(path, skiprows=skipRow(path, key), low_memory=False)
    return df

def skipRowGZ(path):
    skiprows = 0
    with gzip.open(path, 'rb') as f:
        for line in f:
            line = line.decode('utf-8')
            if line[0] != '#':
                break
            skiprows += 1
    skiprows -= 1
    print(skiprows)
    return skiprows

# 读取vcf
def readVcfGZ(path):
    df = pd.read_table(path, skiprows=skipRowGZ(path), low_memory=False)
    return df


# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     ss
   Description :
   Author :        Asdil
   date：          2019/8/16
-------------------------------------------------
   Change Activity:
                   2019/8/16:
-------------------------------------------------
"""
__author__ = 'Asdil'
import os
from Asdil import tool
from multiprocessing import Pool


# 获取染色体数目
def get_chroms(path):
    """
    :param path:  vcf文件 eg: /data7/xxx/abc.vcf 或者/data7/xxx/abc.vcf.gz
    :return:      染色体列表['1', '2', '3', ....]
    """
    suff = tool.splitPath(path)[-2]
    if suff == '.gz':
        # 第一步读取染色体数据
        cmd = f"""gzip -dc {path} """ + """| awk '{print $1}' | grep -v '#' | uniq"""
    else:
        cmd = """awk '{print $1}' """ + f"""{path} |grep -v '#'|uniq"""
    chroms = tool.subprocessPopen(cmd)
    return chroms


# 获取染色体数目，针对已经分染色体的数据
def get_chroms2(path):
    """
    :param path:  已经分过染色体的vcf/vcf.gz文件    eg: /data7/xxx/abc.1.vcf /data7/xxx/abc.2.vcf path=/data7/xxx
    :return: 染色体列表, 分染色体的每个文件路径
    """
    files = tool.getFiles(path, 'vcf')
    chroms = [each.split('.')[-2] for each in files]
    if len(files) == 0:
        files = tool.getFiles(path, 'gz')
        chroms = [each.split('.')[-3] for each in files]
    return chroms, files


# 分割染色体
def split_chroms(path, chroms, out_path, core=8):
    """
    :param path:     vcf路径
    :param chroms:   染色体列表
    :param out_path: 输出文件夹
    :param core:     进程数
    :return:         分染色体的数据，后缀matirx
    """
    name = tool.splitPath(path)[1].split('.')[0]
    cmds = []
    data_list = []
    for chrom in chroms:
        save_path = tool.pathJoin(out_path, f'{name}.{chrom}.matirx')
        data_list.append(save_path)
        suff = tool.splitPath(path)[-2]
        if suff == '.gz':
            cmd = f"""gzip -dc {path} """ + f"""| grep -w '^#CHROM\|^{chrom}' > {save_path}"""
        else:
            cmd = f"""grep -w '^#CHROM\|^{chrom}' {path} > {save_path}"""
        cmds.append(cmd)
    pool = Pool(core)
    pool.map(tool.subprocessCall, cmds)
    pool.close()
    pool.join()
    return data_list


# 分割染色体，已经分染色体的数据
def split_chroms2(paths, out_path, core=8):
    """
    :param paths:     已经分离染色体的文件夹
    :param out_path:  输出文件夹目录
    :param core:      并行核数
    :return:
    """
    data_list = []
    cmds = []
    for path in paths:
        name = tool.splitPath(path)[1]
        suff = tool.splitPath(path)[-2]
        if suff == '.gz':
            chrom = name.split('.')[-2]
            name = name.split('.')[0]
            save_path = tool.pathJoin(out_path, f'{name}.{chrom}.matirx')
            data_list.append(save_path)
            cmd = f"""gzip -dc {path} """ + f"""| grep -w '^#CHROM\|^{chrom}' > {save_path}"""
        else:
            chrom = name.split('.')[-1]
            name = name.split('.')[0]
            save_path = tool.pathJoin(out_path, f'{name}.{chrom}.matirx')
            data_list.append(save_path)
            cmd = f"""grep -w '^#CHROM\|^{chrom}' {path} > {save_path}"""
        cmds.append(cmd)
    pool = Pool(core)
    pool.map(tool.subprocessCall, cmds)
    pool.close()
    pool.join()
    return data_list


# 获取map文件
def get_cpra(paths, out_path, core=8):
    """
    :param paths:     获取染色体 chrom pos ref alt列表
    :param out_path:  输出文件夹
    :param core:      并行核数
    :return:          chrom pos ref alt 文件 .map结尾
    """
    cmds = []
    for path in paths:
        suff = tool.splitPath(path)[-2]
        if suff == '.gz':
            name = tool.splitPath(path)[1].split('.')[0]
            chrom = path.split('.')[-3]
            save_path = tool.pathJoin(out_path, f'{name}.{chrom}.map')
            cmd = f"""gzip -dc {path} | grep -w '^{chrom}' |awk """ + """'{print $1, $2, $4, $5}' >""" + f""" {save_path}"""
        else:
            name = tool.splitPath(path)[1].split('.')[0]
            chrom = path.split('.')[-2]
            save_path = tool.pathJoin(out_path, f'{name}.{chrom}.map')
            cmd = f"""grep -w '^{chrom}' {path}|awk """ + """'{print $1, $2, $4, $5}' >""" + f""" {save_path}"""
        cmds.append(cmd)
    pool = Pool(core)
    pool.map(tool.subprocessCheckAll, cmds)
    pool.close()
    pool.join()


# 转置文件
def trans(params):
    """
    :param params:  文件路径, 输出文件路径
    :return:
    """
    path, out_path = params
    name = tool.splitPath(path)[1]
    save_path = tool.pathJoin(out_path, f'{name}.aped')
    cmd = """awk '{for (i=1; i<=NF; i++) {a[NR,i] = $i}} NF>p { p = NF } END {for(j=1; j<=p; j++) {str=a[1,j]; for(i=2; i<=NR; i++){str=str" "a[i,j];} print str }}' """ + f"{path} > {save_path}"
    tool.subprocessCheckAll(cmd)
    os.remove(path)
    cmd = f"""sed -i '1,9d' {save_path}"""
    tool.subprocessCheckAll(cmd)
    cmd = f"""sed -i 's/0|0/2 2/g' {save_path}"""
    tool.subprocessCheckAll(cmd)
    cmd = f"""sed -i 's/0|1/2 1/g' {save_path}"""
    tool.subprocessCheckAll(cmd)
    cmd = f"""sed -i 's/1|0/1 2/g' {save_path}"""
    tool.subprocessCheckAll(cmd)
    cmd = f"""sed -i 's/1|1/1 1/g' {save_path}"""
    tool.subprocessCheckAll(cmd)
    cmd = f"""sed -i 's/.|./0 0/g' {save_path}"""
    tool.subprocessCheckAll(cmd)
    cmd = f"""sed -i 's/.\/./0 0/g' {save_path}"""
    tool.subprocessCheckAll(cmd)


# 主转置文件
def transpose(paths, out_path, core=8):
    """
    :param paths:     matrix文件路径列表
    :param out_path:  输出文件夹
    :param core:      并行核数
    :return:
    """
    params = [[path, out_path] for path in paths]
    pool = Pool(core)
    pool.map(trans, params)
    pool.close()
    pool.join()


def makeaped(path, out_path, cores=8):
    """
       :path
    """
    chroms = get_chroms(path)
    print('拆分文件...')
    paths = split_chroms(path, chroms, out_path)
    print('拆分文件结束')
    print('生成map文件...')
    get_cpra(paths, out_path, cores)
    print('map文件生成完毕')
    print('转置文件...')
    transpose(paths, out_path, cores)
    print('所有处理完毕')


def makeaped_splited(path, out_path, cores=8):
    chroms, paths = get_chroms2(path)
    print('拆分文件...')
    paths = split_chroms2(paths, out_path, cores)
    print('拆分文件结束')
    print('生成map文件...')
    get_cpra(paths, out_path, cores)
    print('map文件生成完毕')
    print('转置文件...')
    transpose(paths, out_path, cores)
    print('所有处理完毕')


def hp():
    print('1.第一种针对于文件 名称固定 文件名.vcf.gz 或者 文件名.vcf')
    print("path = '/home/jiapeiling/tmp/aa.1.vcf.gz'\nout_path = '/home/jiapeiling/tmp'\nmakeaped(path, out_path)")
    print()
    print('2.第一种针对于已经分染色体的文件 文件夹下包含 文件名.1.vcf.gz 或者 文件名.1.vcf')
    print("path = '/home/jiapeiling/tmp'\nout_path = '/home/jiapeiling/tmp'\nmakeaped_splited(path, out_path)")
    print()
    print('输出文件: 文件名.染色体.map   文件名.染色体.aped')


# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     merge
   Description :
   Author :        Asdil
   date：          2019/9/11
-------------------------------------------------
   Change Activity:
                   2019/9/11:
-------------------------------------------------
"""
__author__ = 'Asdil'
import os
import subprocess
from multiprocessing import Pool


# 合并两个目录
def pathJoin(path1, path2):
    assert type(path1) is str
    assert type(path2) is str
    if path1[-1] != '/':
        path1 += '/'
    if path2[0] == '/':
        path2 = path2[1:]
    return path1 + path2


# 获取目标目录文件
def getFiles(path, extension=None, key=None):
    if extension is not None:
        l = -len(extension)
        ret = [pathJoin(path, each) for each in os.listdir(path) if each[l:] == extension]
    elif key is not None:
        ret = [pathJoin(path, each) for each in os.listdir(path) if key in each]
    else:
        ret = [pathJoin(path, each) for each in os.listdir(path)]
    return ret


def subprocessCall(cmd):
    subprocess.call(cmd, shell=True)


def subprocessCheckAll(cmd):
    subprocess.check_call(cmd, shell=True)


def unzip(path):
    path = pathJoin(path, '*.gz')
    cmd = f'gunzip {path}'
    subprocessCall(cmd)


def delHead(path):
    cmd = f'grep -v "#" {path} > {path}_'
    subprocessCheckAll(cmd)
    return f'{path}_'


def merging(paths, out_path):
    paths = ' '.join(paths)
    cmd = f'cat {paths} > {out_path}'
    subprocessCheckAll(cmd)


def delFiles(paths):
    paths = ' '.join(paths)
    cmd = f'rm -rf {paths}'
    subprocessCheckAll(cmd)


def mergeProcess(paths, out_path):
    if out_path == None:
        out_path = '/'.join(paths[0].split('/')[:-1]) + '/merge.vcf'
    newpaths = [paths[0]]

    for path in paths[1:]:
        path = delHead(path)  # 删除头文件
        newpaths.append(path)

    merging(newpaths, out_path)
    delFiles(newpaths[1:])
    print(f'合并成功,输出文件{out_path}')


def mergeVcf(dir_path, paths=None, out_path=None):
    if paths == None:
        unzip(dir_path)
        paths = getFiles(dir_path, 'vcf')
        chrom = [int(each.split('.')[-2]) for each in paths]  # 默认:  板子号.2.vcf
        paths = list(zip(chrom, paths))
        paths = sorted(paths, key=lambda x: x[0])  # 按照染色体排序
        paths = [each[1] for each in paths]
    mergeProcess(paths, out_path)


def hp():
    print("mergeVcf(dir_path, paths=None, out_path=None)")


# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     aplot
   Description :
   Author :        Asdil
   date：          2019/5/9
-------------------------------------------------
   Change Activity:
                   2019/5/9:
-------------------------------------------------
"""
__author__ = 'Asdil'
import random as rd
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import seaborn as sns
sns.set_style("darkgrid")


# 计算散点平均值方差
def computePosCov(points, nstd=2, ax=None, **kwargs):
    """
    Plots an `nstd` sigma ellipse based on the mean and covariance of a point
    "cloud" (points, an Nx2 array).

    Parameters
    ----------
        points : An Nx2 array of the data points.
        nstd : The radius of the ellipse in numbers of standard deviations.
            Defaults to 2 standard deviations.
        ax : The axis that the ellipse will be plotted on. Defaults to the
            current axis.
        Additional keyword arguments are pass on to the ellipse patch.

    Returns
    -------
        A matplotlib ellipse artist
    """
    pos = points.mean(axis=0)
    cov = np.cov(points, rowvar=False)
    return plotByPosCov(pos, cov, nstd, ax, **kwargs)


# 使用mean，cov计算置信椭圆
def plotByPosCov(pos, cov, nstd=2, ax=None, **kwargs):
    return plotEllipse(cov, pos, nstd, ax, **kwargs)


# 计算置信椭圆
def plotEllipse(cov, pos, nstd=2, ax=None, **kwargs):
    """
    Plots an `nstd` sigma error ellipse based on the specified covariance
    matrix (`cov`). Additional keyword arguments are passed on to the
    ellipse patch artist.

    Parameters
    ----------
        cov : The 2x2 covariance matrix to base the ellipse on
        pos : The location of the center of the ellipse. Expects a 2-element
            sequence of [x0, y0].
        nstd : The radius of the ellipse in numbers of standard deviations.
            Defaults to 2 standard deviations.
        ax : The axis that the ellipse will be plotted on. Defaults to the
            current axis.
        Additional keyword arguments are pass on to the ellipse patch.

    Returns
    -------
        A matplotlib ellipse artist
    """
    def eigsorted(cov):
        vals, vecs = np.linalg.eigh(cov)
        order = vals.argsort()[::-1]
        return vals[order], vecs[:, order]

    if ax is None:
        ax = plt.gca()

    vals, vecs = eigsorted(cov)
    theta = np.degrees(np.arctan2(*vecs[:, 0][::-1]))

    # Width and height are "full" widths, not radius
    width, height = 2 * nstd * np.sqrt(vals)
    ellip = Ellipse(xy=pos, width=width, height=height, angle=theta, **kwargs)
    ellip.set_facecolor('none')
    #     ax.add_artist(ellip)
    ax.add_patch(ellip)
    return ellip, pos[0], pos[1], width, height, theta


# 颜色
def colors(random=False):
    colors = ['red', 'blue', 'green', '#801dae',
              'orange', 'yellow', '#ff7500', '#c3272b',
              '#70f3ff', 'black', '#b36d61', '#4b55c4',
              '#955539', '#3eede7', '#00e079',
              '#c0ebd7', '#eacd76', '#fff143']

    if random:
        rd.shuffle(colors)
    return colors

def hp():
    print('函数 computePosCov 计算散点均值方差画置信椭圆')
    print('函数 plotByPosCov使用均值方差置信椭圆')
    print('函数 plotEllipse画置信椭圆')
    print('函数 color 返回18中颜色')



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     setup
   Description :
   Author :        Asdil
   date：          2018/11/16
-------------------------------------------------
   Change Activity:
                   2019/08/16:
-------------------------------------------------
"""
__author__ = 'Asdil'

from setuptools import setup

setup(name='mofang23',
      version='0.0.1.2',
      description='Tool of Asdil',
      author='Asdil',
      author_email='jpl4job@126.com',
      maintainer='Asdil',
      maintainer_email='jpl4job@126.com',
      license="MIT",
      packages=['mofang23'],
      platforms=["all"],
      url='https://github.com/Asdil/mofang23',
      install_requires=["pandas",
                        "Asdil",
                        "numpy",
                        "matplotlib"],
      classifiers=[
          "Environment :: Web Environment",
          "Intended Audience :: Developers",
          "Operating System :: OS Independent",
          "Topic :: Text Processing :: Indexing",
          "Topic :: Utilities",
          "Topic :: Internet",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Programming Language :: Python",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7", ])

#  python setup.py check
#  python setup.py sdist
#  python setup.py register -r pypi
#  python setup.py sdist upload -r pypi
